#!/usr/bin/env python3

import sys

if sys.version_info.major >= 3:
	from tkinter import *
	from tkinter import ttk
else:
	from Tkinter import *
	import ttk

import eightway2 as eightway

map_data = [
	".........................",
	".##+###+#########+###+##.",
	".#...#...#.....#...#...#.",
	".#...#...#.###.#...#...#.",
	".#...#...#...#.#...#...#.",
	".##.###.##.#.#.##.###.##.",
	".#.........#.#...........",
	".###########.###########.",
	".#.....#.........#.....#.",
	".+.-.-.#.|.|.|.|.#.-.-.+.",
	".#.....#.........#.....#.",
	".+.-.-.#.|.....|.#.-.-.+.",
	".#..........&..........#.",
	".+.-.-.#.|.....|.#.-.-.+.",
	".#.....#.........#.....#.",
	".+.-.-.#.|.|.|.|.#.-.-.+.",
	".#.....#.........#.....#.",
	".###########.###########.",
	"..#.......#...#.......#..",
	"....^.^.^...#...\".\".\".#..",
	"..#.......#...#.......#..",
	"..##+#+#+##...##+#+#+##..",
	"........w.....w..........",
	".\".\".\"....w......w.^.^.^.",
	".=====.w..=====w...=====."
]

colors = {
	"@": "white",
	"&": "gold",
	"#": "firebrick",
	"+": "cyan",
	"-": "gray",
	"=": "brown",
	"|": "gray",
	"w": "green",
	"^": "green",
	'"': "green"
}

compass = ["n", "ne", "e", "se", "s", "sw", "w", "nw"]

camera = eightway.Camera(0, -4, -6, 600)
scene = eightway.Scene()

scene.addSprite("@", 0, 0)

for y in range(len(map_data)):
	for x in range(len(map_data[y])):
		if map_data[y][x] != ".":
			# Invert the y because in game coordinates
			# it goes north, into the distance, instead.
			scene.addSprite(map_data[y][x], x - 12, 12 - y)

#scene.viewDist = 12
scene.spacing["u"] = 0.8
scene.center["y"] = -5

player = scene.sprites[0]
player.y = scene.center["y"]
facing = 0

top = Tk()
top.title("ASCII Manor")

viewport = Canvas(top, background="black", width=1024, height=600)
viewport.pack(fill="both", expand=1)

def render():
	w, h = viewport.winfo_width(), viewport.winfo_height()
	camera.f = min(w, h)
	center_x = w / 2
	viewport.delete("tile")
	for sprite in scene.render(camera, compass[facing]):
		tile_font = "courier " + str(int(sprite.scale)) + " bold"
		viewport.create_text(
			int(sprite.x + center_x), int(sprite.y),
			text=sprite.sid,
			font=tile_font, fill=colors[sprite.sid],
			anchor="s", tags="tile")

def turn_left():
	global facing
	facing = (facing - 1) % 8
	render()

def turn_right():
	global facing
	facing = (facing + 1) % 8
	render()

def walk_forward():
	if compass[facing] == "n":
		player.y += 1
	elif compass[facing] == "ne":
		player.x += 1
		player.y += 1
	elif compass[facing] == "e":
		player.x += 1
	elif compass[facing] == "se":
		player.x += 1
		player.y -= 1
	elif compass[facing] == "s":
		player.y -= 1
	elif compass[facing] == "sw":
		player.x -= 1
		player.y -= 1
	elif compass[facing] == "w":
		player.x -= 1
	elif compass[facing] == "nw":
		player.x -= 1
		player.y += 1
		
	scene.center["x"] = player.x
	scene.center["y"] = player.y
	render()

top.bind("<Expose>", lambda e: render())
top.bind("<Control-r>", lambda e: render())
top.bind("<F5>", lambda e: render())

top.bind("<Key-Up>", lambda e: walk_forward())
top.bind("<Key-Left>", lambda e: turn_left())
top.bind("<Key-Right>", lambda e: turn_right())

top.mainloop()
